@@ -27,7 +27,7 @@ char* blacklist[] = { "xournal", "gimp-2.6", "mypaint", "inkscape" };
 int blacklistLength = 4;
 
 /* Profiles */
-int profileCount = 5;
+int profileCount = 6;
 Profile profiles[] = {
 				{ 	.windowClass = "evince",
 					.scrollInherit = 0,
@@ -136,8 +136,36 @@ Profile profiles[] = {
 					.rotateLeftAction = { ACTIONTYPE_KEYPRESS, XK_Left, MODIFIER_CONTROL },
 					.rotateRightAction = { ACTIONTYPE_KEYPRESS, XK_Right, MODIFIER_CONTROL },
 					.tapInherit = 1
-				}
+				},
+			 	{	.windowClass = "SimCity 4.exe",
+								.scrollInherit = 0,
+								.scrollMinDistance = 20,
+								.hscrollStep = 1000,
+								.vscrollStep = 1000,
+								.scrollBraceAction = { ACTIONTYPE_BUTTONPRESS, 3, 0	},
+								.scrollUpAction = { ACTIONTYPE_NONE,0,0 },
+								.scrollDownAction = { ACTIONTYPE_NONE,0,0 },
+								.scrollLeftAction = { ACTIONTYPE_NONE,0,0 },
+								.scrollRightAction = { ACTIONTYPE_NONE,0,0 },
+//								.scrollMinDistance = 20,
+//								.hscrollStep = 10,
+//								.vscrollStep = 10,
+//								.scrollBraceAction = { ACTIONTYPE_NONE,0,0 },
+//								.scrollUpAction = { ACTIONTYPE_KEYPRESS, XK_Down, 0 },
+//								.scrollDownAction = { ACTIONTYPE_KEYPRESS, XK_Up, 0 },
+//								.scrollLeftAction = { ACTIONTYPE_KEYPRESS, XK_Right, 0 },
+//								.scrollRightAction = { ACTIONTYPE_KEYPRESS, XK_Left, 0 },
+								.zoomInherit = 0,
+								.zoomMinDistance = 80,
+								.zoomInAction = { ACTIONTYPE_BUTTONPRESS, 4, 0 },
+								.zoomOutAction = { ACTIONTYPE_BUTTONPRESS, 5, 0 },
+								.zoomStep = 1.5,
+								.rotateInherit = 1,
+								.tapInherit = 1
+							}
 			};
+/* DON'T FORGET TO CHANGE profileCount WHEN ADDING PROFILES! */
+
 
 /* Default (fallback) profile */
 Profile defaultProfile = {	.windowClass = NULL,
