@@ -26,6 +26,7 @@
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
 #include <X11/extensions/XInput2.h>
+#include <X11/extensions/Xrandr.h>
 #include "twofingemu.h"
 #include "profiles.h"
 #include <unistd.h>
@@ -35,7 +36,7 @@
 #define EXIT_SUCCESS 0
 #define EXIT_FAILURE 1
 
-int debugmode = 0;
+int debugMode = 0;
 /* Clickmode: 0 - first finger; 1 - second finger; 2 - center */
 int clickmode = 2;
 
@@ -100,7 +101,7 @@ FingerInfo fingerInfos[2] = { { .x=0, .y=0, .down = 0, .id = 0 }, { .x=0, .y=0,
 
 /* X stuff */
 Display* display;
-Window ROOT;
+Window root;
 int screenNum;
 int deviceID;
 Atom WM_CLASS;
@@ -109,12 +110,11 @@ Time fairlyCurrentTime;
 
 /* Calibration data */
 int calibMinX, calibMaxX, calibMinY, calibMaxY;
+double calibFactorX, calibFactorY;
 unsigned char calibSwapX, calibSwapY, calibSwapAxes;
 
-/* The real width and height of the display, in pixels */
-unsigned int realDisplayWidth, realDisplayHeight;
-/* The width and height of the (possibly) rotated display as the user perceives it, in pixels */
-unsigned int displayWidth, displayHeight;
+/* The width and height of the screen in pixels */
+unsigned int screenWidth, screenHeight;
 
 /* Finger data */
 int fingersDown = 0;
@@ -162,9 +162,8 @@ int invalidWindowHandler(Display *dsp, XErrorEvent *err) {
 /* Grab the device so input is captured */
 void grab(Display* display, int grabDeviceID) {
 	XIEventMask device_mask;
-	unsigned char mask_data[1];
+	unsigned char mask_data[1] = { 0 };
 	mask_data[0] = 1; /* To prevent warning */
-	device_mask.deviceid = grabDeviceID;
 	device_mask.mask_len = sizeof(mask_data);
 	device_mask.mask = mask_data;
 	XISetMask(device_mask.mask, XI_ButtonPress);
@@ -173,7 +172,7 @@ void grab(Display* display, int grabDeviceID) {
 
 	XIGrabModifiers modifiers[1] = { { 0, 0 } };
 
-	XIGrabButton(display, grabDeviceID, 1, ROOT, None, GrabModeAsync,
+	XIGrabButton(display, grabDeviceID, 1, root, None, GrabModeAsync,
 			GrabModeAsync, False, &device_mask, 1, modifiers);
 
 }
@@ -181,7 +180,7 @@ void grab(Display* display, int grabDeviceID) {
 /* Ungrab the device so input can be handled by application directly */
 void ungrab(Display* display, int grabDeviceID) {
 	XIGrabModifiers modifiers[1] = { { 0, 0 } };
-	XIUngrabButton(display, grabDeviceID, 1, ROOT, 1, modifiers);
+	XIUngrabButton(display, grabDeviceID, 1, root, 1, modifiers);
 
 }
 
@@ -212,7 +211,7 @@ void releaseButton() {
 	}
 }
 
-/* Executes the given action -- synthesizes key/button press, release or both, depenging
+/* Executes the given action -- synthesizes key/button press, release or both, depending
  * on value of whatToDo (EXECUTEACTION_PRESS/_RELEASE/_BOTH). */
 void executeAction(Action* action, int whatToDo) {
 	if (whatToDo & EXECUTEACTION_PRESS) {
@@ -240,10 +239,12 @@ void executeAction(Action* action, int whatToDo) {
 		switch (action->actionType) {
 		case ACTIONTYPE_BUTTONPRESS:
 			XTestFakeButtonEvent(display, action->keyButton, True, CurrentTime);
+			XFlush(display);
 			break;
 		case ACTIONTYPE_KEYPRESS:
 			XTestFakeKeyEvent(display, XKeysymToKeycode(display,
 					action->keyButton), True, CurrentTime);
+			XFlush(display);
 			break;
 		}
 
@@ -254,10 +255,12 @@ void executeAction(Action* action, int whatToDo) {
 		switch (action->actionType) {
 		case ACTIONTYPE_BUTTONPRESS:
 			XTestFakeButtonEvent(display, action->keyButton, False, CurrentTime);
+			XFlush(display);
 			break;
 		case ACTIONTYPE_KEYPRESS:
 			XTestFakeKeyEvent(display, XKeysymToKeycode(display,
 					action->keyButton), False, CurrentTime);
+			XFlush(display);
 			break;
 		}
 
@@ -308,26 +311,38 @@ Window getCurrentWindow() {
 	int i = 0;
 	while (1) {
 		i++;
-		if(i >= 5) return None;
-		if (currentWindow == ROOT) {
+		if(i >= 5) {
+			XFree(classHint);
+			return None;
+		}
+		if (currentWindow == root) {
 			/* No top-level window available. Should never happen. */
 			XFree(classHint);
 			return currentWindow;
 		}
 
 		if (XGetClassHint(display, currentWindow, classHint) == 0) {
-			XQueryTree(display, currentWindow, &root, &parent, &childWindows,
-					&childCount);
+			/* Has no WM_CLASS, thus no top-level window */
+			if(XQueryTree(display, currentWindow, &root, &parent, &childWindows,
+					&childCount)) {
 
-			if (childWindows != NULL)
-				XFree(childWindows);
+				if (childWindows != NULL)
+					XFree(childWindows);
 
-			if(currentWindow == parent) {
-				/* something wrong */
+				if(currentWindow == parent) {
+					/* something wrong */
+					XFree(classHint);
+					return currentWindow;
+				}
+				/* Continue with parent until we find WM_CLASS */
+				currentWindow = parent;
+			} else {
+				XFree(classHint);
 				return currentWindow;
 			}
-			currentWindow = parent;
 		} else {
+			if(classHint->res_class != NULL) XFree(classHint->res_class);
+			if(classHint->res_name != NULL) XFree(classHint->res_name);
 			XFree(classHint);
 			return currentWindow;
 		}
@@ -344,23 +359,31 @@ Profile* getWindowProfile(Window w) {
 		/* Read WM_CLASS member */
 		if (XGetClassHint(display, w, classHint)) {
 
+			if(debugMode) {
+				printf("Current window: '%s'\n", classHint->res_name);
+			}
+
 			int i;
 			/* Look for the profile with this class */
 			for (i = 0; i < profileCount; i++) {
 				if (strncmp(classHint->res_name, profiles[i].windowClass, 30)
 						== 0) {
+					if(classHint->res_class != NULL) XFree(classHint->res_class);
+					if(classHint->res_name != NULL) XFree(classHint->res_name);
 					XFree(classHint);
 					/* Return this profile */
 					return &profiles[i];
 				}
 			}
 
+			if(classHint->res_class != NULL) XFree(classHint->res_class);
+			if(classHint->res_name != NULL) XFree(classHint->res_name);
 			XFree(classHint);
 			/* No profile found, return default. */
 			return &defaultProfile;
 		} else {
-			XFree(classHint);
 			/* Couldn't get WM_CLASS, return default profile. */
+			XFree(classHint);
 			return &defaultProfile;
 		}
 	} else {
@@ -393,6 +416,12 @@ void processFingers() {
 
 		/* Get current profile */
 		currentProfile = getWindowProfile(getCurrentWindow());
+		if(debugMode) {
+			if(currentProfile->windowClass != NULL) {
+				printf("Use profile '%s'\n", currentProfile->windowClass);
+			}
+		}
+
 
 		/* If there had already been a single-touch event raised because the
 		 * user was too slow, stop it now. */
@@ -413,6 +442,7 @@ void processFingers() {
 		/* Move pointer to center between touch points */
 		XTestFakeMotionEvent(display, -1, gestureStartCenterX,
 				gestureStartCenterY, CurrentTime);
+		XFlush(display);
 
 	} else if (TWO_FINGERS_ON) {
 		/* Moved with two fingers */
@@ -437,6 +467,7 @@ void processFingers() {
 			/* Move pointer to center between touch points */
 			XTestFakeMotionEvent(display, -1, currentCenterX, currentCenterY,
 					CurrentTime);
+			XFlush(display);
 		}
 
 		/* Perform gestures as long as there are some. */
@@ -466,6 +497,7 @@ void processFingers() {
 				/* Assume first finger is at ID 0 and second finger at ID 1, might have to be changed later */
 				XTestFakeMotionEvent(display, -1, fingerInfos[clickmode].x, fingerInfos[clickmode].y,
 						CurrentTime);
+				XFlush(display);
 			}
 
 			if (currentProfile->tapInherit) {
@@ -538,16 +570,17 @@ int checkGesture() {
 	int xdist = currentCenterX - gestureStartCenterX;
 	int ydist = currentCenterY - gestureStartCenterY;
 	double moveDist = sqrt(xdist * xdist + ydist * ydist);
-	if (moveDist > maxDist)
+	if (moveDist > maxDist && fingersDown == 2)
 		maxDist = moveDist;
 
 	/* We don't know yet what to do, so look if we can decide now. */
-	if (amPerformingGesture == GESTURE_UNDECIDED) {
+	if (amPerformingGesture == GESTURE_UNDECIDED && fingersDown == 2) {
 		int scrollMinDist = currentProfile->scrollMinDistance;
 		if (currentProfile->scrollInherit)
 			scrollMinDist = defaultProfile.scrollMinDistance;
 		if ((int) moveDist > scrollMinDist) {
 			amPerformingGesture = GESTURE_SCROLL;
+			if(debugMode) printf("Start scrolling gesture\n");
 
 			if (currentProfile->scrollInherit) {
 				executeAction(&(defaultProfile.scrollBraceAction),
@@ -576,6 +609,7 @@ int checkGesture() {
 			zoomMinDist = defaultProfile.zoomMinDistance;
 		if (abs((int) currentDist - gestureStartDist) > zoomMinDist) {
 			amPerformingGesture = GESTURE_ZOOM;
+			if(debugMode) printf("Start zoom gesture\n");
 			return 1;
 		}
 
@@ -594,6 +628,7 @@ int checkGesture() {
 		if (abs(rotatedBy) > rotateMinAngle && (int) currentDist
 				> rotateMinDist) {
 			amPerformingGesture = GESTURE_ROTATE;
+			if(debugMode) printf("Start rotation gesture\n");
 			return 1;
 		}
 	}
@@ -670,6 +705,7 @@ int checkGesture() {
 		if (currentProfile->zoomInherit)
 			zoomStep = defaultProfile.zoomStep;
 		if (zoomedBy > zoomStep) {
+			if(debugMode) printf("Zoom in step\n");
 			if (currentProfile->zoomInherit) {
 				executeAction(&(defaultProfile.zoomInAction),
 						EXECUTEACTION_BOTH);
@@ -681,6 +717,7 @@ int checkGesture() {
 			gestureStartDist = gestureStartDist * zoomStep;
 			return 1;
 		} else if (zoomedBy < 1 / zoomStep) {
+			if(debugMode) printf("Zoom out step\n");
 			if (currentProfile->zoomInherit) {
 				executeAction(&(defaultProfile.zoomOutAction),
 						EXECUTEACTION_BOTH);
@@ -704,6 +741,7 @@ int checkGesture() {
 		if (currentProfile->rotateInherit)
 			rotateStep = defaultProfile.rotateStep;
 		if (rotatedBy > rotateStep) {
+			if(debugMode) printf("Rotate right\n");
 			if (currentProfile->rotateInherit) {
 				executeAction(&(defaultProfile.rotateRightAction),
 						EXECUTEACTION_BOTH);
@@ -714,6 +752,7 @@ int checkGesture() {
 
 			gestureStartAngle = gestureStartAngle + rotateStep;
 		} else if (rotatedBy < -rotateStep) {
+			if(debugMode) printf("Rotate left\n");
 			if (currentProfile->rotateInherit) {
 				executeAction(&(defaultProfile.rotateLeftAction),
 						EXECUTEACTION_BOTH);
@@ -740,7 +779,7 @@ void leaveWindow() {
 	}
 }
 
-/* Called when a blacklisted window is focussed */
+/* Called when a blacklisted window is focused */
 void enterBlacklistedWindow() {
 	//printf("Entered bad window.\n");
 	activateAtRelease = 0;
@@ -755,16 +794,22 @@ int isWindowBlacklisted(Window w) {
 	XClassHint* classHint = XAllocClassHint();
 
 	if (XGetClassHint(display, w, classHint)) {
-		printf("Found window with id %i and class '%s' \r\n", (int) w,
-				classHint->res_name);
+		if(debugMode) {
+			printf("Found window with id %i and class '%s' \n", (int) w,
+					classHint->res_name);
+		}
 
 		for (i = 0; i < blacklistLength; i++) {
 			if (strncmp(classHint->res_name, blacklist[i], 30) == 0) {
+				if(classHint->res_class != NULL) XFree(classHint->res_class);
+				if(classHint->res_name != NULL) XFree(classHint->res_name);
 				XFree(classHint);
 				return 1;
 			}
 		}
 
+		if(classHint->res_class != NULL) XFree(classHint->res_class);
+		if(classHint->res_name != NULL) XFree(classHint->res_name);
 	}
 	XFree(classHint);
 	return 0;
@@ -791,15 +836,29 @@ void windowMapped(Window w) {
 
 }
 
+void setScreenSize(XRRScreenChangeNotifyEvent * evt) {
+	screenWidth = evt->width;
+	screenHeight = evt->height;
+	if(calibMaxX != calibMinX && calibMaxY != calibMinY) {
+		calibFactorX = ( screenWidth / (double) (calibMaxX - calibMinX));
+		calibFactorY = ( screenHeight / (double) (calibMaxY - calibMinY));
+	} else {
+		calibFactorX = 1.;
+		calibFactorY = 1.;
+	}
+	if(debugMode) {
+		printf("New screen size: %i x %i\n", screenWidth, screenHeight);
+	}
+}
+
 /* X thread that processes X events in parallel to kernel device loop */
 void* xLoopThreadFunction(void* arg) {
 	XEvent ev;
-
 	while (1) {
 		XNextEvent(display, &ev);
 
-		if (XGetEventData(display, (XGenericEventCookie *) &ev)) {
-			XGenericEventCookie *cookie = &ev.xcookie;
+		if (XGetEventData(display, &(ev.xcookie))) {
+			XGenericEventCookie *cookie = &(ev.xcookie);
 			if (cookie->evtype == XI_Motion) {
 				/* Move event */
 				XIDeviceEvent* data = cookie->data;
@@ -808,6 +867,7 @@ void* xLoopThreadFunction(void* arg) {
 					/* Fake single-touch move event */
 					XTestFakeMotionEvent(display, -1, (int) data->root_x,
 							(int) data->root_y, CurrentTime);
+					XFlush(display);
 				}
 			} else if (cookie->evtype == XI_PropertyEvent) {
 				/* Device properties changed -> recalibrate. */
@@ -815,7 +875,7 @@ void* xLoopThreadFunction(void* arg) {
 				readCalibrationData(0);
 			}
 
-			XFreeEventData(display, (XGenericEventCookie *) &ev);
+			XFreeEventData(display, &(ev.xcookie));
 
 		} else {
 			if (ev.type == MapNotify) {
@@ -827,11 +887,9 @@ void* xLoopThreadFunction(void* arg) {
 				/* This should only be called for blacklisted windows, but it's sometimes also
 				 * called when a non-blacklisted window is left. But that's no problem for us. */
 				leaveWindow();
-				//			} else if(ev.type == ConfigureNotify) {
-				//				displayWidth = DisplayWidth(display, screenNum);
-				//				displayHeight = DisplayHeight(display, screenNum);
-				//				printf("Display width: %i; height: %i\n", displayWidth, displayHeight);
-
+			} else if(ev.type == 101) {
+				/* Why isn't this magic constant explained anywhere?? */
+				setScreenSize((XRRScreenChangeNotifyEvent *) &ev);
 			}
 		}
 	}
@@ -841,66 +899,106 @@ void* xLoopThreadFunction(void* arg) {
  * Registers enter/leave event handlers for blacklisted windows. Executes them for
  * the current window. */
 void checkRunningWindows() {
-	Window root;
+	Window aroot;
 	Window parent;
 	Window* childWindows = NULL;
 	unsigned int childCount;
 	/* Get all top-level windows */
-	XQueryTree(display, ROOT, &root, &parent, &childWindows, &childCount);
+	if(XQueryTree(display, root, &aroot, &parent, &childWindows, &childCount)) {
 
-	if (childWindows != NULL) {
-		int i;
-		for (i = 0; i < childCount; i++) {
-			windowMapped(childWindows[i]);
+		if (childWindows != NULL) {
+			int i;
+			for (i = 0; i < childCount; i++) {
+				windowMapped(childWindows[i]);
+			}
+			XFree(childWindows);
 		}
-		XFree(childWindows);
 	}
 }
 
 /* Reads the calibration data from evdev, should be self-explanatory. */
 void readCalibrationData(int exitOnFail) {
-
+	if(debugMode) {
+		printf("Start calibration\n");
+	}
 	Atom retType;
 	int retFormat;
 	unsigned long retItems, retBytesAfter;
 	unsigned int* data;
-	XIGetProperty(display, deviceID, XInternAtom(display,
+	if(XIGetProperty(display, deviceID, XInternAtom(display,
 			"Evdev Axis Calibration", 0), 0, 4 * 32, False, XA_INTEGER,
 			&retType, &retFormat, &retItems, &retBytesAfter,
-			(unsigned char**) &data);
+			(unsigned char**) &data) != Success) {
+		data = NULL;
+	}
 
-	if (retItems != 4 || data[0] == data[1] || data[2] == data[3]) {
+	if (data == NULL || retItems != 4 || data[0] == data[1] || data[2] == data[3]) {
 		/* evdev might not be ready yet after resume. Let's wait a second and try again. */
 		sleep(1);
 
-		XIGetProperty(display, deviceID, XInternAtom(display,
+		if(XIGetProperty(display, deviceID, XInternAtom(display,
 				"Evdev Axis Calibration", 0), 0, 4 * 32, False, XA_INTEGER,
 				&retType, &retFormat, &retItems, &retBytesAfter,
-				(unsigned char**) &data);
+				(unsigned char**) &data) != Success) {
+			return;
+		}
 
 		if (retItems != 4 || data[0] == data[1] || data[2] == data[3]) {
 
-			if (exitOnFail) {
-				printf(
-						"No valid calibration data found. Evdev Axis Calibration not set?\n");
-				exit(1);
-			} else {
-				return;
+			if(debugMode) {
+				printf("No calibration data found, use default values.\n");
 			}
-		}
-	}
-	calibMinX = data[0];
-	calibMaxX = data[1];
-	calibMinY = data[2];
-	calibMaxY = data[3];
 
-	XFree(data);
+			/* Get minimum/maximum of axes */
+
+			int nDev;
+			XIDeviceInfo * deviceInfo = XIQueryDevice(display, deviceID, &nDev);
+
+			int c;
+			for(c = 0; c < deviceInfo->num_classes; c++) {
+				if(deviceInfo->classes[c]->type == XIValuatorClass) {
+					XIValuatorClassInfo* valuatorInfo = (XIValuatorClassInfo *) deviceInfo->classes[c];
+					if(valuatorInfo->mode == XIModeAbsolute) {
+						if(valuatorInfo->label == XInternAtom(display, "Abs X", 0)) {
+							calibMinX = valuatorInfo->min;
+							calibMaxX = valuatorInfo->max;
+						} else if(valuatorInfo->label == XInternAtom(display, "Abs Y", 0)) {
+							calibMinY = valuatorInfo->min;
+							calibMaxY = valuatorInfo->max;
+						}
+					}
+				}
+			}
+
+			XIFreeDeviceInfo(deviceInfo);
+
+		} else {
+			calibMinX = data[0];
+			calibMaxX = data[1];
+			calibMinY = data[2];
+			calibMaxY = data[3];
+		}
+	} else {
+		calibMinX = data[0];
+		calibMaxX = data[1];
+		calibMinY = data[2];
+		calibMaxY = data[3];
+	}
+	calibFactorX = ( screenWidth / (double) (calibMaxX - calibMinX));
+	calibFactorY = ( screenHeight / (double) (calibMaxY - calibMinY));
+	printf("Calibration factors: %f %f\n", calibFactorX, calibFactorY);
+
+	if(data != NULL) {
+		XFree(data);
+	}
 
 	unsigned char* data2;
 
-	XIGetProperty(display, deviceID, XInternAtom(display,
+	if(XIGetProperty(display, deviceID, XInternAtom(display,
 			"Evdev Axis Inversion", 0), 0, 2 * 8, False, XA_INTEGER, &retType,
-			&retFormat, &retItems, &retBytesAfter, (unsigned char**) &data2);
+			&retFormat, &retItems, &retBytesAfter, (unsigned char**) &data2) != Success) {
+		return;
+	}
 
 	if (retItems != 2) {
 		if (exitOnFail) {
@@ -916,10 +1014,12 @@ void readCalibrationData(int exitOnFail) {
 
 	XFree(data2);
 
-	XIGetProperty(display, deviceID,
+	if(XIGetProperty(display, deviceID,
 			XInternAtom(display, "Evdev Axes Swap", 0), 0, 8, False,
 			XA_INTEGER, &retType, &retFormat, &retItems, &retBytesAfter,
-			(unsigned char**) &data2);
+			(unsigned char**) &data2) != Success) {
+		return;
+	}
 
 	if (retItems != 1) {
 		if (exitOnFail) {
@@ -930,13 +1030,6 @@ void readCalibrationData(int exitOnFail) {
 		}
 	}
 	calibSwapAxes = data2[0];
-	if (calibSwapAxes) {
-		displayWidth = realDisplayHeight;
-		displayHeight = realDisplayWidth;
-	} else {
-		displayWidth = realDisplayWidth;
-		displayHeight = realDisplayHeight;
-	}
 
 	XFree(data2);
 
@@ -953,7 +1046,7 @@ int main(int argc, char **argv) {
 	for (i = 1; i < argc; i++) {
 		if (strcmp(argv[i], "--debug") == 0) {
 			doDaemonize = 0;
-			debugmode = 1;
+			debugMode = 1;
 		} else if (strcmp(argv[i], "--wait") == 0) {
 			doWait = 1;
 		} else if (strcmp(argv[i], "--click=first") == 0) {
@@ -991,15 +1084,15 @@ int main(int argc, char **argv) {
 	/* Read X data */
 	screenNum = DefaultScreen(display);
 
-	ROOT = RootWindow(display, screenNum);
+	root = RootWindow(display, screenNum);
 
-	realDisplayWidth = DisplayWidth(display, screenNum);
-	realDisplayHeight = DisplayHeight(display, screenNum);
+//	realDisplayWidth = DisplayWidth(display, screenNum);
+//	realDisplayHeight = DisplayHeight(display, screenNum);
 
 	WM_CLASS = XInternAtom(display, "WM_CLASS", 0);
 
 	/* Get notified about new windows */
-	XSelectInput(display, ROOT, StructureNotifyMask | SubstructureNotifyMask);
+	XSelectInput(display, root, StructureNotifyMask | SubstructureNotifyMask);
 
 	//TODO load blacklist and profiles from file(s)
 
@@ -1031,21 +1124,46 @@ int main(int argc, char **argv) {
 
 		XSetErrorHandler(invalidWindowHandler);
 
-		/* XInput Extension available? */
+
 		int opcode, event, error;
+		if (!XQueryExtension(display, "RANDR", &opcode, &event,
+				&error)) {
+			printf("X RANDR extension not available.\n");
+			XCloseDisplay(display);
+			exit(1);
+		}
+
+		/* Which version of XRandR? We support 1.3 */
+		int major = 1, minor = 3;
+		if (!XRRQueryVersion(display, &major, &minor)) {
+			printf("XRandR version not available.\n");
+			XCloseDisplay(display);
+			exit(1);
+		} else if(!(major>1 || (major == 1 && minor >= 3))) {
+			printf("XRandR 1.3 not available. Server supports %d.%d\n", major, minor);
+			XCloseDisplay(display);
+			exit(1);
+		}
+
+		/* XInput Extension available? */
 		if (!XQueryExtension(display, "XInputExtension", &opcode, &event,
 				&error)) {
 			printf("X Input extension not available.\n");
+			XCloseDisplay(display);
 			exit(1);
 		}
 
 		/* Which version of XI2? We support 2.0 */
-		int major = 2, minor = 0;
+		major = 2; minor = 0;
 		if (XIQueryVersion(display, &major, &minor) == BadRequest) {
 			printf("XI2 not available. Server supports %d.%d\n", major, minor);
+			XCloseDisplay(display);
 			exit(1);
 		}
 
+		screenWidth = XDisplayWidth(display, screenNum);
+		screenHeight = XDisplayHeight(display, screenNum);
+
 		int n;
 		XIDeviceInfo *info = XIQueryDevice(display, XIAllDevices, &n);
 		if (!info) {
@@ -1086,7 +1204,10 @@ int main(int argc, char **argv) {
 		device_mask2.mask_len = sizeof(mask_data2);
 		device_mask2.mask = mask_data2;
 		XISetMask(device_mask2.mask, XI_PropertyEvent);
-		XISelectEvents(display, ROOT, &device_mask2, 1);
+		XISelectEvents(display, root, &device_mask2, 1);
+
+		/* Recieve events when screen size changes */
+		XRRSelectInput(display, root, RRScreenChangeNotifyMask);
 
 		/* Needed for XTest to work correctly */
 		XTestGrabControl(display, True);
@@ -1099,7 +1220,7 @@ int main(int argc, char **argv) {
 		}
 
 		/* Needed for some reason to receive events */
-		XGrabPointer(display, ROOT, False, 0, GrabModeAsync, GrabModeAsync,
+		XGrabPointer(display, root, False, 0, GrabModeAsync, GrabModeAsync,
 				None, None, CurrentTime);
 		XUngrabPointer(display, CurrentTime);
 
@@ -1128,38 +1249,43 @@ int main(int argc, char **argv) {
 						if (index == 0 || index == 1) {
 							fingerInfos[index].id = tempFingerInfo.id;
 
-							/* Calibrate coordinate */
+							/* Calibrate coordinates */
 							if (calibSwapAxes) {
-								fingerInfos[index].x = ((tempFingerInfo.y
-										- calibMinY) * displayWidth)
-										/ (calibMaxY - calibMinY);
-								fingerInfos[index].y = ((tempFingerInfo.x
-										- calibMinX) * displayHeight)
-										/ (calibMaxX - calibMinX);
+
+								fingerInfos[index].x = (int)
+										((tempFingerInfo.y - calibMinX) * calibFactorX);
+								fingerInfos[index].y = (int)
+										((tempFingerInfo.x - calibMinY) * calibFactorY);
+
 							} else {
 
-								fingerInfos[index].x = ((tempFingerInfo.x
-										- calibMinX) * displayWidth)
-										/ (calibMaxX - calibMinX);
-								fingerInfos[index].y = ((tempFingerInfo.y
-										- calibMinY) * displayHeight)
-										/ (calibMaxY - calibMinY);
+								fingerInfos[index].x = (int)
+										((tempFingerInfo.x - calibMinX) * calibFactorX);
+								fingerInfos[index].y = (int)
+										((tempFingerInfo.y - calibMinY) * calibFactorY);
+
+//								fingerInfos[index].x = ((tempFingerInfo.x
+//										- calibMinX) * displayWidth)
+//										/ (calibMaxX - calibMinX);
+//								fingerInfos[index].y = ((tempFingerInfo.y
+//										- calibMinY) * displayHeight)
+//										/ (calibMaxY - calibMinY);
 							}
 							if (calibSwapX)
-								fingerInfos[index].x = displayWidth
+								fingerInfos[index].x = screenWidth
 										- fingerInfos[index].x;
 							if (calibSwapY)
-								fingerInfos[index].y = displayHeight
+								fingerInfos[index].y = screenHeight
 										- fingerInfos[index].y;
 
 							if (fingerInfos[index].x < 0)
 								fingerInfos[index].x = 0;
 							if (fingerInfos[index].y < 0)
 								fingerInfos[index].y = 0;
-							if (fingerInfos[index].x > displayWidth)
-								fingerInfos[index].x = displayWidth;
-							if (fingerInfos[index].y > displayHeight)
-								fingerInfos[index].y = displayHeight;
+							if (fingerInfos[index].x > screenWidth)
+								fingerInfos[index].x = screenWidth;
+							if (fingerInfos[index].y > screenHeight)
+								fingerInfos[index].y = screenHeight;
 						}
 						fingerInfos[index].down = 1;
 					} else if (0 == ev[i].code) { // Ev_Sync event end
